//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file adc.c
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  ADC support
//!
//! ADC support functions
//!
//! \author David R Brooks
//!
//! \version 17/09/2007 DRB initial design and coding
//!
//! \attention           [insert attention note such as "Maintain <A HREF="/////Alpha/Electronics/Documentation/mydoc.doc"> documentation </a>"]
//! \warning             [insert warnings or delete line]
//! \note                [insert notes or delete line]
//! \sa                  [insert see-also files or delete line]
//!
//! \todo                [insert todo list or delete line]
//! \bug                 [insert bug list or delete line]
//---------------------------------------------------------------------------
// 2008.08.17 14:13  [YYYY.MM.DD] Last edited (CodeWright)

//-Included files -----------------------------------------------------------
#include "main.h"

//-Local Symbols ------------------------------------------------------------
// [insert local #defines and variable types]

//-Globals ------------------------------------------------------------------

sint2 ScaledAdc_s2[NUMADCINPS];			// Adjusted ADC results
sint2 OffsetAdc_s2[NUMADCINPS] = {0};	// Direct offsets, if any

//-Locals -------------------------------------------------------------------
typedef struct 
{
	bool1	UseOffset : 1;
    uint1	ExtMux	  : 2;
    uint1	IntMux	  : 3;
    sint2	Offset;
    uint2	Scale;
} T_ADCIN;

const T_ADCIN AdcParm[NUMADCINPS] flash = {
	{FALSE, 0, 0,     0,   295},  	//   IV2L mV
    {TRUE,  1, 0,	  0,  4133},  	// ICLIPR mA Clip filter*
    {TRUE,  2, 0,	  0, 21843},  	//  IDIRR mA Main filter
    {FALSE, 3, 0,     0,   295},  	//   IV2R mV
    {FALSE, 0, 1,     0,    32},  	//    SW1  V (scaled to 0/1 at half-span)
    {FALSE, 1, 1,     0,    32},  	//    SW2  V  "
    {FALSE, 2, 1,     0,    32},  	//    SW3  V  "
    {FALSE, 3, 1,     0,     0},  	//   CLIP mV (reads directly)
    {FALSE, 0, 2, -4050,  6226},  	//   M330  V
    {FALSE, 1, 2,     0, 11154},  	//   P660  V
    {FALSE, 2, 2, -4010,  3146},  	//   M140  V
    {FALSE, 3, 2,     0,  3244},  	//   P140  V
    {TRUE,  0, 3,     0,  4133},  	// ICLIPL mA Clip filter*
    {TRUE,  1, 3,     0, 21843},  	//  IDIRL mA Main filter
    {FALSE, 2, 3, -2730,  6554},  	//   TEMP dC
    {FALSE, 3, 3,     0,  6423},  	//   P330  V
    {FALSE, 0, 6, -2048,   800},  	//  VOUTL mV
    {FALSE, 0, 7, -2048,   800}     //  VOUTR mV
    };
    
//	*Note: Clip settings =27053 if R343 & R440 are removed (as per schematic)
//                   Use   4133 if resistors are in.
    
static uint1 adc_ch;
static T_ADCIN chdata;    

//-Local prototypes ---------------------------------------------------------
//! \cond SEE_PROTOTYPE

// [insert local protoypes]

//! \endcond

//-Local Functions ----------------------------------------------------------

//--AdcGetSpec-----------------------------------------------------------------
//! Retrieve ADC channel parameters
//-----------------------------------------------------------------------------
//! The channel table is in Flash, & needs to be copied to RAM for use
//! 
//! \param chan       Channel to read
//! \param pDest      Ptr. to destination buffer
//! 
//-----------------------------------------------------------------------------
static inline void AdcGetSpec(uint1 chan, T_ADCIN *pDest)
{
    memcpy_P(pDest, &AdcParm[chan], sizeof(T_ADCIN));
}

//--AdcStartCycle--------------------------------------------------------------
//! Set the ADC input selectors, start ADC
//-----------------------------------------------------------------------------
//! Fetch the current channel data, set the multiplexers
//! The channel data is static: it will be used to scale the reading
//!
//! \param first         Set channel = 0, start of a conversion block
//! 
//-----------------------------------------------------------------------------
void AdcStartCycle(bool1 first)
{
	if(first)
    {
    	adc_ch = 0;
    }
	AdcGetSpec(adc_ch, &chdata);		// Static: we use it after as well
    EXMUX_ADC(chdata.ExtMux);
    ADMUX   = chdata.IntMux;
	ADCSRA |= BV(ADSC);					// Start the next ADC cycle
}

//-Global Functions ---------------------------------------------------------

//--SIGNAL---------------------------------------------------------------------
//! ADC operation-complete interrupt
//-----------------------------------------------------------------------------
//! The ADC fires this interrupt after each conversion.
//! Don't run the ADC autonomously, as it gets nasty making sure channels are
//! updated	at the proper time. This way, it's safe.
//! All the ADC inputs are heavily filtered externally, so there's no need for
//! a fast response here. The chosen Tconv=208uS gives a full cycle of about 
//! 4mS, which is the slowest possible. This still gives several cycles per
//! main state-machine cycle, so the ADC is driven from the main timer,
//! asynchronous to the state machine.
//!	.
//! The ADC reference is 4096mV, so the 10-bit ADC reading is multiplied by 4,
//! giving a reading in mV.
//!	.
//!	After logging a Fault, the only ADC readings updated are the 3 Switch
//! inputs (to allow the switch to work). The others are frozen, so holding
//! the failed readings for inspection.
//!
//-----------------------------------------------------------------------------
SIGNAL(SIG_ADC)
{
	if((!InFault()) || ((adc_ch>=SW1) && (adc_ch<=SW3)))
    {
		sint4 adc = ADC << 2;				// Get the 16-bit ADC, scale by 4 (ie mV)
	    
	    adc += chdata.Offset;
	    if(chdata.Scale != 0)				// Scaling reqd.?
	    {
		    adc *= chdata.Scale;			// Result scaled 65536
		    adc /= 65536;					// Just reads the high bytes
	    }
	    if(chdata.UseOffset)
	    {
	    	adc -= OffsetAdc_s2[adc_ch];	// Discount the idle-state reading
	    } 
	    ScaledAdc_s2[adc_ch] = adc;
    }
    adc_ch++;							    // Next channel
    if(adc_ch < NUMADCINPS)
    {									    // Run another conversion
    	AdcStartCycle(FALSE);
    }
}
