//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file adc.h
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Publicly accessible functions and types for adc.c
//!
//! ADC support
//!
//! \author David R Brooks
//! \date 17/09/2007
//!
//---------------------------------------------------------------------------
// 2008.06.21 12:12  [YYYY.MM.DD] Last edited (CodeWright)

#ifndef _ADC__H
#define _ADC__H

//-Included files ------------------------------------------------------------
#include "typedefn.h"

//-Symbols -------------------------------------------------------------------

typedef enum				// ADC channel IDs (Int:Ext)
{
	IV2R	 = 0,
	ICLIPR,
	IDIRR,
	IV2L,
	SW1,
	SW2,
	SW3,
	CLIP,
	M330,
	P660,
	M140,
	P140,
	ICLIPL,
	IDIRL,
	TEMP,					// Deg-K * 10mV
	P330,
	VOUTL,
	VOUTR,
	NUMADCINPS				// Size, must be last
} E_ADC_CHAN;

//-Global Externs ------------------------------------------------------------

extern sint2 ScaledAdc_s2[NUMADCINPS];		// Raw ADC results
extern sint2 OffsetAdc_s2[NUMADCINPS];		// Direct offsets, if any

//-Global Function Prototypes ------------------------------------------------

//--AdcInit--------------------------------------------------------------------
//! ADC initialisation (XTAL / 128) = 62.5kHz. Tconv = 208uS
//-----------------------------------------------------------------------------
//! Set up the ADC, but don't start
//! 
//! \pre              Initialise IO ports first
//! 
//-----------------------------------------------------------------------------
static inline void AdcInit(void){ADCSRA = BV(ADEN)| BV(ADIE) | BV(ADPS2) | BV(ADPS1) | BV(ADPS0);};

//--AdcFixOffsets--------------------------------------------------------------
//! Save offsets recorded at start-up
//-----------------------------------------------------------------------------
//! Some ADC inputs (typically the output current monitors) are subject to
//! variable offsets, which are detected at start-up, and discounted.
//! This fn. is called when the HT is on, just before the output valves are
//! biased on, & records selected ADC values as "really zero".
//! 
//-----------------------------------------------------------------------------
static inline void AdcFixOffsets(void){memcpy(OffsetAdc_s2, ScaledAdc_s2, sizeof(ScaledAdc_s2));};

//--AdcStartCycle--------------------------------------------------------------
//! Set the ADC input selectors, start ADC
//-----------------------------------------------------------------------------
//! Fetch the current channel data, set the multiplexers
//! The channel data is static: it will be used to scale the reading
//!
//! \param first         Set channel = 0, start of a conversion block
//! 
//-----------------------------------------------------------------------------
void AdcStartCycle(bool1 first);

#endif //_ADC__H

