//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file calrec.h
//!
//! \brief  Publicly accessible functions and types for calrec.c
//!
//! Header for non-volatile calibration data
//! See calrec.c for the default values
//!
//! \author David R Brooks
//! \date 28/01/2008
//!
//---------------------------------------------------------------------------


#ifndef _CALREC__H
#define _CALREC__H

//-Symbols -------------------------------------------------------------------
typedef struct					// Limiting values for supply voltages
{
	sint2	minv;
    sint2	maxv;
} T_LIMITS;

typedef struct					// Controls for one servo loop
{
	uint4   	dacdrive;		// DAC setting (output)
    uint2   	dacscale;		// Gain * 1/65536
    sint2		setpoint;		// Setpoint
} T_SERVO;

typedef struct					// Calibration record
{
	uint2		baud;			// Baud-rate
    uint2       iv1_10[2];		// V1 cathode current (mA*10)
    uint2       iv2[2];			// V2 cathode current (mA)
    							
    T_SERVO		servo[4];		// Output servo controls				
    uint2       fastscale;		// Scale factor for quick gain (calibn.)
    T_LIMITS	calvolt;		// OP mV must be within this, at start	
    T_LIMITS	calamp;			// OP mA must be within this, at start	
    T_SERVO		clip;			// Clip-voltage driver					
    uint1		period_ms;		// State-machine period (mS)			
    uint2   	startdelay_us;	// Startup delay (for UART to stabilise)
    T_LIMITS	vlp660;			// Limits for +660
    T_LIMITS	vlp330;			// Limits for +330						
    T_LIMITS	vlp140;			// Limits for +140						
    T_LIMITS	vlm140;			// Limits for -140						
    T_LIMITS	vlm330;			// Limits for -330
    uint2       IoutMax;		// Max. filtered output current
    						
	uint2		sumcheck;
} T_CALIBN;

typedef enum {FIRST, SECOND, BOTH} CalWrite;

//-Global Externs ------------------------------------------------------------
extern T_CALIBN CalRam;			// Public calibration data

//-Global Function Prototypes ------------------------------------------------

//--CalibrationDefault---------------------------------------------------------
//! Reset default calibration data
//-----------------------------------------------------------------------------
//! [insert detailed description]
//! 
//-----------------------------------------------------------------------------
void CalibrationDefault(void);

//--CalibrationRead------------------------------------------------------------
//! Read EEROM calibration to RAM
//-----------------------------------------------------------------------------
//! Read the EEROM copy. If the checksum is OK, copy it to RAM.
//! Try the first EEROM copy, then the second. The first good one is accepted.
//! Otherwise, leave the RAM copy unchanged (ie default initialisation).
//! If one copy is bad, overwrite it with the good one.
//! 
//! \return           TRUE if one EEROM was OK, & was copied
//! 
//-----------------------------------------------------------------------------
bool1 CalibrationRead(void);

//--CalibrationWrite-----------------------------------------------------------
//! Store RAM calibration in EEROM
//-----------------------------------------------------------------------------
//! There are 2 copies (for safety) in EEROM. Write the first, then the second.
//! If power fails while writing the first, the second still holds the old
//! data.
//! 
//-----------------------------------------------------------------------------
void  CalibrationWrite(CalWrite bank);

#endif //_CALREC__H

