//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file log.c
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Logging functions
//!
//! [insert detailed decription]
//!
//! \author David R Brooks
//!
//! \version 17/09/2007 DRB initial design and coding
//!
//---------------------------------------------------------------------------
// 2008.08.19 18:04  [YYYY.MM.DD] Last edited (CodeWright)

//-Included files -----------------------------------------------------------
#include "main.h"

//-Local Symbols ------------------------------------------------------------
#define TOPWRD(x)	((uint2*)&(x))[1]

//-Globals ------------------------------------------------------------------

//-Locals -------------------------------------------------------------------
static uint1 LogPtr  = 100;
static uint1 EditPtr = 0;			// Indicate the item to be edited

// NB The designations Left & Right on the log screen refer to the amplifier
//    upright, viewed from the front (same as the channel designations).
//    The internal signal name suffixes L & R are reversed, ie viewed from
//    underneath (don't ask why...)

const S_LOGITEM LogName[] flash =	// List of items for the log display
{  //  Data                   Name       Fmt
   // ------------------------------------------------------------------
	{ &ScaledAdc_s2[P660],   	 		 		"+660V",     S2},	// Rail voltage:   +660
	{ &ScaledAdc_s2[P330],   	 		 		"+330V",     S2},	//  			   +330
	{ &ScaledAdc_s2[P140],   	 		 		"+140V",     S2},	//  			   +140
	{ &ScaledAdc_s2[M140],   	 		 		"-140V",     S2},	//  			   -140
	{ &ScaledAdc_s2[M330],   	 		 		"-330V",     S2},	//  			   -330
//
	{ &CalRam.vlp660.maxv,  	 	 	 		"+660 Max", ES2},	// Voltage limits: +660 Max
    { &CalRam.vlp330.maxv,   			 		"+330 Max", ES2},	//  			   +330 Max
    { &CalRam.vlp140.maxv,  			 		"+140 Max", ES2},	//  			   +140 Max
    { &CalRam.vlm140.maxv,  			 		"-140 Max", ES2},	//  			   -140 Max
    { &CalRam.vlm330.maxv,   			 		"-330 Max", ES2},	//  			   -330 Max
//    									 		    
	{ &CalRam.vlp660.minv,  	 	 	 		"+660 Min", ES2},	// Voltage limits: +660 Min
    { &CalRam.vlp330.minv,  	 	 	 		"+330 Min", ES2},	//  			   +330 Min
    { &CalRam.vlp140.minv,   			 		"+140 Min", ES2},	// 				   +140 Min
    { &CalRam.vlm140.minv,   			 		"-140 Min", ES2},	// 				   -140 Min
    { &CalRam.vlm330.minv,  	 	 	 		"-330 Min", ES2},	//  			   -330 Min
//    									 		    
	{ &CalRam.fastscale,  		 	 	 		"Acq  Scl", EU2},	// Fast acquisition scaling
    { &CalRam.calvolt.minv,  			 		"Acq Vmin", ES2},	// Acquisition limit: Min Volts
    { &CalRam.calvolt.maxv,  			 		"Acq Vmax", ES2},	// 					  Max
    { &CalRam.calamp.minv,  			 		"Acq Imin", ES2},	// 					  Min mA
    { &CalRam.calamp.maxv,  			 		"Acq Imax", ES2},	// 					  Max
//    											    
	{ &seconds,    		 	 	 		 		"Elapsed",   U2},	//
    { &PowerCycle,       	 	 		 		"AC Line",   U2},	//
	{ &CalRam.clip.setpoint,					"Clip Set", EU2},	// Clip-level setpoint
    { &TOPWRD(CalRam.clip.dacdrive),            "Clip PWM",  U2},	// Clip output
	{ &ScaledAdc_s2[CLIP],   	 		 		"Clip mV",   S2},	// Clip feedback mV
//
    { &SwitchMsg,          	 	 		 		"Switch",    CP},	//
	{ &ScaledAdc_s2[IV2R],   	 		 		"L  V2 mA",  S2},	// mA
	{ &ScaledAdc_s2[IDIRR],  	 		 		"L  Idir",   S2},	// mA
	{ &ScaledAdc_s2[IV2L],   	 		 		"R  V2 mA",  S2},	// mA
	{ &ScaledAdc_s2[IDIRL],  	 		 		"R  Idir",   S2},	// mA
//    
	{ &cstate.name,    		 	 		 		"State",     CS},	// 
	{ &ScaledAdc_s2[VOUTR],  	 		 		"L  Vout",   S2},	// mV		  
	{ &ScaledAdc_s2[ICLIPR], 	 		 		"L  Iclip",  S2},	// mA		  
	{ &ScaledAdc_s2[VOUTL],  	 		 		"R  Vout",   S2},	// mV 
	{ &ScaledAdc_s2[ICLIPL], 	 		 		"R  Iclip",  S2},	// mA 
//    																	  
	{ &FaultMsg,  		 	 	 		 		"Fault",     CP},	//	  
    { &CalRam.servo[RIGHT_HI].setpoint,  		"LU Set V", EU2},	// Setpoint: V
    { &CalRam.servo[RIGHT_LO].setpoint,  		"LL Set I", EU2},	//  		 mA 
    { &CalRam.servo[LEFT_HI ].setpoint,  		"RU Set V", EU2},	//  		 V 
    { &CalRam.servo[LEFT_LO ].setpoint,  		"RL Set I", EU2},	//  		 mA 
//    
//	{ &CalRam.period_ms,  	 	 		 		"Updte mS",  U1},	// State machine period, mS
    { &PulseDelay,                              "SSR Phse",  U1},   // SSR phase setting
    { &DacDelta[RIGHT_HI],   	 		 		"LU Error",  S2},	//
    { &DacDelta[RIGHT_LO],   	 		 		"LL Error",  S2},	//  
    { &DacDelta[LEFT_HI ],   	 		 		"RU Error",  S2},	//  
    { &DacDelta[LEFT_LO ],   	 		 		"RL Error",  S2},	//  
//    
	{ &ScaledAdc_s2[TEMP],   	 		 		"Temp C",    S2},	// PCB temp. C
    { &CalRam.servo[RIGHT_HI].dacscale,  		"LU Gain",  EU2},	// Gain scaling
    { &CalRam.servo[RIGHT_LO].dacscale,  		"LL Gain",  EU2},	//  
    { &CalRam.servo[LEFT_HI ].dacscale,  		"RU Gain",  EU2},	//  
    { &CalRam.servo[LEFT_LO ].dacscale,  		"RL Gain",  EU2},	//  
//    
//	{ &CalRam.startdelay_us,  		 	 		"Strt Del",  U2},	// Start delay, uS
    { &IV1[LEFT],                               "L IV1x10",  U2},
    { &TOPWRD(CalRam.servo[RIGHT_HI].dacdrive), "LU Accum", TU2},	// 32-bit acc. (MS word) 
    { &TOPWRD(CalRam.servo[RIGHT_LO].dacdrive), "LL Accum", TU2},	//  
    { &TOPWRD(CalRam.servo[LEFT_HI ].dacdrive), "RU Accum", TU2},	//  
    { &TOPWRD(CalRam.servo[LEFT_LO ].dacdrive), "RL Accum", TU2},	//  
//    
//	{ &CalRam.baud,  		 	 		 		"BaudRate", EU2},	// Baud rate (to 57600)
    { &IV1[RIGHT],                              "R IV1x10",  U2},
    { &DacValue[RIGHT_HI],   	 		 		"LU DAC",    U2},	// DAC units 
    { &DacValue[RIGHT_LO],   	 		 		"LL DAC",    U2},	//  
    { &DacValue[LEFT_HI ],   	 		 		"RU DAC",    U2},	//  
    { &DacValue[LEFT_LO ],   	 		 		"RL DAC",    U2},	//  
};

#define LOGITEMS  (sizeof(LogName) / sizeof(S_LOGITEM))

//-Local prototypes ---------------------------------------------------------
//! \cond SEE_PROTOTYPE

// [insert local protoypes]

//! \endcond

//-Local Functions ----------------------------------------------------------
// [insert local functions]

//-Global Functions ---------------------------------------------------------

//--RunningLog-----------------------------------------------------------------
//! Use to get a running output
//-----------------------------------------------------------------------------
//! For testing only.
//! Customise this fn. to get a running output of selected variables.
//! Capture the output to a log-file for later analysis.
//! Use commas as field separators, if importing into a spreadsheet.
//! 
//-----------------------------------------------------------------------------
void RunningLog(void)
{
	static uint2 s = 0;		// Cycle count
    
	outc('\n');
    outn(s, 6, 10, FALSE);	// Cycle count (ie * 20mS)
    outc(',');
    outs(cstate.name);		// Controller state
    outc(',');
    outn(ScaledAdc_s2[IDIRL],  6, 10, TRUE);
    outc(',');
    outn(ScaledAdc_s2[ICLIPL], 6, 10, TRUE);
}


//--GetLogSpace----------------------------------------------------------------
//! Return no. of lines used by log
//-----------------------------------------------------------------------------
//! Get spave occupied on screen by the log values
//! 
//! \return           No. of log lines
//! 
//-----------------------------------------------------------------------------
uint2 GetLogSpace(void)		// Return no. of lines used by log
{
	return LOGITEMS / ITEMSPERROW;
}

//--LogInit--------------------------------------------------------------------
//! Initialise the log-value counter
//-----------------------------------------------------------------------------
//! Sets up to start a new display cycle
//! 
//-----------------------------------------------------------------------------
void LogInit(void)
{
	LogPtr = 100;			// After the end, so it will reload
}

//--ShowNextLogItem------------------------------------------------------------
//! Display next item in sequence
//-----------------------------------------------------------------------------
//! Called every state cycle, displays the next item from the log table above.
//! 
//-----------------------------------------------------------------------------
void ShowNextLogItem(void)
{
    uint2 	   value;
	S_LOGITEM  logline;
    sint1      radix   = 10;
    bool1      sign    = FALSE;
    bool1      canedit = FALSE;
    bool1      string  = FALSE;
    uint1      pad     = ' ';
    
    memcpy_P(&logline, &LogName[LogPtr], sizeof(logline));
    
	if(LogPtr >= LOGITEMS)			// End of the list
	{
		LogPtr = 0;
		SetCursor(LOGROW, 1);
    	memcpy_P(&logline, &LogName[LogPtr], sizeof(logline));
	}

    switch(logline.dtype)
    {
    case  U1:
    	value  = (uint2)(*(uint1*)logline.pItem);
    	value &= 0x0ff;
        break;
    case EU2:
    	canedit = TRUE;				// Then fall through
    case TU2:
    case  U2:
    	value  = *(uint2*)logline.pItem;
        break;
    case ES2:
    	canedit = TRUE;				// Then fall through
    case  S2:
    	sign   = TRUE;
    	value  = *(uint2*)logline.pItem;
        break;
    case  X1:
    	radix  = 16;
    	value  = *(uint1*)logline.pItem;
    	value &= 0x0ff;
    	break;
    case  X2:
    	radix  = 16;
    	value  = *(uint2*)logline.pItem;
    	break;
    case  CS:
    	string = TRUE;
		outsw(logline.sName, LOGWIDTH - NUMWIDTH - 3);
        outs((uint1*)logline.pItem);
        break;
    case  CP:
    	string = TRUE;
		outsw(logline.sName, LOGWIDTH - NUMWIDTH - 2);
        outsw_P(*(PGM_P*)logline.pItem, 6);
        break;
    default:
    	value = 0;
        break;
    }   

	if(!string)
    {    
		outsw(logline.sName, LOGWIDTH - NUMWIDTH - 2);
		outn(value, NUMWIDTH, radix, sign);
    }
    
    if(LogPtr == EditPtr)
    {
    	if(canedit)
        {
        	if(HalfSec)
            {
				pad = '<';
            }
        }
        else
        {
        	EditMove(1);		// Can't edit here: move on
        }	
    }
    outc(pad);
	LogPtr++;
}

//--EditMove-------------------------------------------------------------------
//! Move the Edit Pointer on the display
//-----------------------------------------------------------------------------
//! NB: You can't move direct to END, if it's not editable - the pointer will
//!     advance circularly to the next editable, ie same as HOME.
//! 
//! \param offset     Amount to move by
//! 
//-----------------------------------------------------------------------------
void EditMove(sint1 offset)
{
	sint1 t;
    
	switch(offset)
    {
    case 100:					// Home
    	EditPtr = 0;
        break;
    default:
		t = EditPtr + offset;
		
	    if(t < 0)
	    {
	    	t += LOGITEMS;
	    }
	    else if(t >= LOGITEMS)
	    {
	    	t -= LOGITEMS;
	    }
	    EditPtr = (uint1)t;
    }
}

//--EditNum--------------------------------------------------------------------
//! Adjust calibration value. EditPtr will point at an editable item.
//-----------------------------------------------------------------------------
//! Response to F1..F10 keys: change a calibration setting, in RAM only.
//! 
//! \param offset     Amount to adjust by
//! 
//-----------------------------------------------------------------------------
void EditNum (sint2 offset)
{
	S_LOGITEM  logline;
    sint4 t;
    uint2 u;
    sint2 s;
    
    memcpy_P(&logline, &LogName[EditPtr], sizeof(logline));
    switch(logline.dtype)
    {
    case EU2:							// 2-byte unsigned
    	u = *(uint2*)logline.pItem;
        t = u;
        t += offset;
        if((t <= MAX_UINT2) && (t >= 0))
        {								// In range
        	*(uint2*)logline.pItem = (uint2)(t & 0xffff);
        }
    	break;
    case ES2:							// 2-byte signed
    	s = *(sint2*)logline.pItem;
        t = s;
        t += offset;					// In range
        if((t <= MAX_SINT2) && (t >= MIN_SINT2))
        {
        	*(sint2*)logline.pItem = (sint2)(t & 0xffff);
        }
    	break;
    default:							// Others are not editable
    	return;
    }
}

