//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file ancomp.h
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Publicly accessible functions and types for ancomp.c
//!
//! Analogue comparator: AC line interrupts
//!
//! \author David R Brooks
//! \date 18/10/2007
//!
//---------------------------------------------------------------------------
// 2008.08.16 12:11  [YYYY.MM.DD] Last edited (CodeWright)

#ifndef __ANCOMP_H
#define __ANCOMP_H

//-Included files ------------------------------------------------------------
#include "typedefn.h"

//-Symbols -------------------------------------------------------------------

//-Global Externs ------------------------------------------------------------
extern uint2 PowerCycle;		// TC0 counts per AC cycle (128 or 160)
extern uint1 SsrMask;			// Short-pulse the SSR outputs
extern uint1 PulseDelay;		// SSR pulse delay

//-Global Function Prototypes ------------------------------------------------
	// Inputs AC1, AC2. Comparator interrupts on both edges.
static inline void  AnCompInit(void){SFIOR = 0x00; ACSR = 0x08;};

//--IsAcLineReady--------------------------------------------------------------
//! Check if the AC frequency measurement is complete
//-----------------------------------------------------------------------------
//! 
//! \return           TRUE if done
//! 
//-----------------------------------------------------------------------------
bool1  IsAcLineReady(void);

//--SetPhase-------------------------------------------------------------------
//! Set the conduction angles for the AC lines
//-----------------------------------------------------------------------------
//! Angle is expressed as a fraction of 256 (0=off, 255=full-on). Only one
//! circuit may have a partial phase at any one time: the other must be 0 or 255.
//! 
//! \param HtAngle      Phase for HT circuit
//! \param LtAngle      Phase for LT circuit
//!
//! \return           TRUE if successful
//! 
//-----------------------------------------------------------------------------
bool1  SetPhase(uint1 HtAngle, uint1 LtAngle);

#endif //__ANCOMP_H
