//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file spi.h
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Publicly accessible functions and types for spi.c
//!
//! Support for SPI (DAC & output register)
//!
//! \author David R Brooks
//! \date 27/10/2007
//!
//---------------------------------------------------------------------------
// 2008.08.16 15:37  [YYYY.MM.DD] Last edited (CodeWright)


#ifndef __SPI_H
#define __SPI_H

//-Included files ------------------------------------------------------------
#include "typedefn.h"

//-Symbols -------------------------------------------------------------------
#define SPISETUP (BV(SPIE) | BV(SPE) | BV(MSTR) | BV(SPR1) | BV(SPR0))

typedef enum{G_OFF, G_WIDE, G_NARROW} PidState;

//-Global Externs ------------------------------------------------------------
								// NB Update DacValue atomically!
extern uint2 DacValue[4];		// Value to send to TLV5604 DAC
extern uint1 Reg595;			// Value to send to 74HC595 output bits
extern sint2 DacDelta[4];		// Servo error terms

//-Global Function Prototypes ------------------------------------------------

//--SpiInit--------------------------------------------------------------------
//! Start the SPI subsystem
//-----------------------------------------------------------------------------
//! Triggers a SPI cycle: the interrupts then run continuously
//! 
//-----------------------------------------------------------------------------
static inline void SpiInit(void){SPCR = SPISETUP; SPDR = 0;};

//--DacUpdate------------------------------------------------------------------
//! Update the DAC drive values from the errors
//-----------------------------------------------------------------------------
//! For each OP grid, compute & update the bias value.
//! 
//-----------------------------------------------------------------------------
void DacUpdate(PidState fast);

//--DacSetValue---------------------------------------------------------------------
//! Set new value on given DAC channel
//-----------------------------------------------------------------------------
//! The value will update on the next loop of the SPI engine.
//! 
//! \param chan       Channel, 0-3
//! \param val        Value 0-1023
//! 
//! \warning          Run from base level: interrupts are re-enabled
//! 
//-----------------------------------------------------------------------------
void DacSetValue(uint1 chan, uint2 val);

#endif //__SPI_H

