//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file drivers.c
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Valve amplifier control. Device drivers
//!
//! Low-level IO drivers & hardware virtualisation for the Igor-II control.
//! This code can be used with any high-level strategy.
//!
//! \author David R Brooks
//!
//! \version 18/06/2007 DRB initial design and coding
//!
//! \attention           CPU: Atmel ATmega08
//! \note                Assumes 8MHz clock (internal)
//!
//---------------------------------------------------------------------------
// 2008.07.26 13:13  [YYYY.MM.DD] Last edited (CodeWright)

//-Included files -----------------------------------------------------------
#include "main.h"

//-Globals ------------------------------------------------------------------

//-Locals -------------------------------------------------------------------

//-Local prototypes ---------------------------------------------------------
//! \cond SEE_PROTOTYPE

// [insert local protoypes]

//! \endcond

//-Local Functions ----------------------------------------------------------
// [insert local functions]

//-Global Functions ---------------------------------------------------------

//--TimebaseInit---------------------------------------------------------------
//! Set up the main state-machine timer (T2)
//-----------------------------------------------------------------------------
//! [insert detailed description]
//! 
//! \param period_u1  Update period (mS)
//!
//! \note             System clock = 8MHz, prescaler = 1024 giving a nominal 8kHz timebase
//! 
//-----------------------------------------------------------------------------
void TimebaseInit(uint1 period_u1)
{
    TCCR2  = BV(WGM21) | BV(CS22) | BV(CS21) | BV(CS20);   // Prescaler: CLK/1024
    OCR2   = period_u1 * CLKSCALE;              // Nominal clock is 8kHz
    TIMSK |= BV(OCIE2);                         // Interrupt on
}

//--PwmInit--------------------------------------------------------------------
//! Start the PWM (clip) voltage from OC1A
//-----------------------------------------------------------------------------
//! OC1A generates a PWM signal, which is integrated (by an external active 
//! LPF) to develop the clip-voltage for the current monitors.
//! 
//-----------------------------------------------------------------------------
void PwmInit(void)
{
    TCCR1A = BV(COM1A1) | BV(WGM11) | BV(WGM10);
    TCCR1B = BV(WGM12)  | BV(CS10);
    SetClipPwm(CalRam.clip.dacdrive >> ACCUMSHIFT);
}

//--ClipUpdate-----------------------------------------------------------------
//! Update the clip-voltage PWM generator
//-----------------------------------------------------------------------------
//! Call periodically, to track the clip-voltage generator
//! This feedback loop INVERTS: as per the external circuit
//!
//-----------------------------------------------------------------------------
void ClipUpdate(void)
{
    sint4 diff;
//            Measurement              Setpoint
	diff = ScaledAdc_s2[CLIP] - CalRam.clip.setpoint;
	CalRam.clip.dacdrive -= diff * CalRam.clip.dacscale;
    SetClipPwm(CalRam.clip.dacdrive >> ACCUMSHIFT);
}

