//WARNING comment syntax is important,  if unsure, check out http://www.doxygen.org/manual.html

//---------------------------------------------------------------------------
//! \file drivers.h
//!
//! \project Igor-IIL 25+25W OTL valve amplifier
//!
//! \brief  Publicly accessable functions and types for drivers.c
//!
//! \project Valve amplifier control
//!
//! \author David R Brooks
//! \date 18/06/2007
//!
//! \attention           CPU: Atmel ATmega08
//!
//---------------------------------------------------------------------------
// 2008.06.28 11:04  [YYYY.MM.DD] Last edited (CodeWright)

#ifndef __DRIVERS_H
#define __DRIVERS_H

//-------------------------------------------------------------------------
// Symbols
//-------------------------------------------------------------------------

//-Global Externs ------------------------------------------------------------


//-Global Function Prototypes ------------------------------------------------

//--TimebaseInit---------------------------------------------------------------
//! Set up the main state-machine timer (T2)
//-----------------------------------------------------------------------------
//! [insert detailed description]
//! 
//! \param period_u1  Update period (mS)
//!
//! \note             System clock = 8MHz, prescaler = 1024 giving a nominal 8kHz timebase
//! 
//-----------------------------------------------------------------------------
void TimebaseInit(uint1 period_u1);

//--PwmInit--------------------------------------------------------------------
//! Start the PWM (clip) voltage from OC1A
//-----------------------------------------------------------------------------
//! OC1A generates a PWM signal, which is integrated (by an external active 
//! LPF) to develop the clip-voltage for the current monitors.
//! 
//-----------------------------------------------------------------------------
void PwmInit(void);

//--SetClipPwm-----------------------------------------------------------------
//! Set duty cycle for the clip-voltage PWM
//-----------------------------------------------------------------------------
//! The output-current clip level is generated by a PWM (OC1A) & an active
//! low-pass filter. Resolution is 10 bits.
//! NB The useful output values are quite low. The clip voltage is greater by
//! 1 diode drop than the clip-filter output. The current sensor transfer
//! function is 3V/A, so a 0.5A clip (corresponding to a 0.25A quiescent) means
//! a clip level of 1.5V, and a filter output of 1500-700 = 800mV. This occurs
//! at a PWM around 140. It may be worth attenuating the clip-filter input.
//! 
//! \param duty_u2    Duty cycle value
//! 
//-----------------------------------------------------------------------------
static inline void SetClipPwm(uint2 duty_u2){OCR1A = duty_u2;};

//--ClipUpdate-----------------------------------------------------------------
//! Update the clip-voltage PWM generator
//-----------------------------------------------------------------------------
//! Call periodically, to track the clip-voltage generator
//! 
//-----------------------------------------------------------------------------
void ClipUpdate(void);

#endif //__DRIVERS_H

